package ociinstaller

import (
	"context"
	"fmt"
	"path/filepath"
	"time"
	"log"

	"github.com/turbot/steampipe/constants"
	versionfile "github.com/turbot/steampipe/ociinstaller/versionfile"
)

// InstallFdw installs the Steampipe Postgres foreign data wrapper from an OCI image
func InstallFdw(ctx context.Context, dbLocation string) (string, error) {
	tempDir := NewTempDir(dbLocation)
	defer func() {
		if err := tempDir.Delete(); err != nil {
			log.Printf("[TRACE] Failed to delete temp dir '%s' after installing fdw: %s", tempDir, err)
		}
	}()

	imageDownloader := NewOciDownloader()

	// download the blobs.
	image, err := imageDownloader.Download(ctx, NewSteampipeImageRef(constants.FdwImageRef), ImageTypeFdw, tempDir.Path)
	if err != nil {
		return "", err
	}

	// install the files
	if err = installFdwFiles(image, tempDir.Path, dbLocation); err != nil {
		return "", err
	}

	if err := updateVersionFileFdw(image); err != nil {
		return string(image.OCIDescriptor.Digest), err
	}

	return string(image.OCIDescriptor.Digest), nil
}

func updateVersionFileFdw(image *SteampipeImage) error {
	timeNow := versionfile.FormatTime(time.Now())
	v, err := versionfile.LoadDatabaseVersionFile()
	if err != nil {
		return err
	}
	v.FdwExtension.Version = image.Config.Fdw.Version
	v.FdwExtension.Name = "fdwExtension"
	v.FdwExtension.ImageDigest = string(image.OCIDescriptor.Digest)
	v.FdwExtension.InstalledFrom = image.ImageRef.requestedRef
	v.FdwExtension.LastCheckedDate = timeNow
	v.FdwExtension.InstallDate = timeNow
	return v.Save()
}

func installFdwFiles(image *SteampipeImage, tempdir string, dest string) error {
	hubBinPath := filepath.Join(dest, "lib", "postgresql")
	hubControlPath := filepath.Join(dest, "share", "postgresql", "extension")
	hubSQLPath := filepath.Join(dest, "share", "postgresql", "extension")

	fileName := image.Fdw.BinaryFile
	sourcePath := filepath.Join(tempdir, fileName)
	if _, err := ungzip(sourcePath, hubBinPath); err != nil {
		return fmt.Errorf("could not unzip %s to %s", sourcePath, hubBinPath)
	}

	fileName = image.Fdw.ControlFile
	sourcePath = filepath.Join(tempdir, fileName)
	if err := moveFileWithinPartition(sourcePath, filepath.Join(hubControlPath, fileName)); err != nil {
		return fmt.Errorf("could not install %s to %s", sourcePath, hubControlPath)
	}

	fileName = image.Fdw.SqlFile
	sourcePath = filepath.Join(tempdir, fileName)
	if err := moveFileWithinPartition(sourcePath, filepath.Join(hubSQLPath, fileName)); err != nil {
		return fmt.Errorf("could not install %s to %s", sourcePath, hubSQLPath)
	}
	return nil
}
